/* 
 *  gstalker stock charter
 * 
 *  Copyright (c) 1998 Stefan S. Stratigakos
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */


#include "myheader.h"
#include "pixmaps.c"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>


/*** error messages ***/
char cant_create_tempfile[] = {"Hey! I can't create any temporary files! I'm outta here..."};
char cant_create_chart[] = {"Hey! I can't create any new chart files! I'm outta here..."};
char error_notify[] = {"Hey! There were some errors during the import. Check the error log for the details pal.."};
char cant_open_errorlog[] = {"Hey! I can't open my error log! I'm outta here..."};
char cant_open_importfile[] = {"Hey! I can't open the file to import! I'm outta here..."};
char nothing_selected[] = {"You gotta select something first pal..."};
char invalid_selection[] = {"Nope, try another selection..."};
char already_exists[] = {"No way, that selection is already there..."};
char nothing_entered[] = {"You gotta enter something first pal..."};
char invalid_entry[] = {"Whatever you entered was bogus, try again pal..."};
char nothing_selected_edit[] = {"You gotta select something to insert first pal..."};
char nothing_selected_edit2[] = {"You gotta select a place to insert pal..."};
char cant_open_group[] = {"Whoa! For some reason, I can't open that group..."};
char group_empty[] = {"The group you selected is empty, there's nothing to see here..."};
char cant_open_chart[] = {"Hey! For some reason, I can't open that chart..."};
char cant_create[] = {"Whoa! For some reason, I can't create that file..."};
char invalid_date[] = {"Whatever you entered for date has got some bogus stuff in there..."};
char invalid_date2[] = {"You gotta have at least 8 characters for the date pal..."};
char invalid_date3[] = {"You gotta put a date in there first pal..."};
char data_missing[] = {"Hey, my data directory is either empty or mising! That's it, I'm outta here..."};
char group_missing[] = {"Hey, my group directory is either empty or missing! That's it, I'm outta here..."};
char invalid_moving[] = {"Whatever you entered for moving average value was bogus, try again pal..."};
char invalid_pnc[] = {"Whatever you entered for net change was bogus, try again pal..."};
char invalid_pvc[] = {"Whatever you entered for volume net change was bogus, try again pal..."};
char invalid_threshold[] = {"Whatever you entered for threshold was bogus, try again pal..."};
char cant_delete_last_record[] = {"Can't delete the only record left."};


/*** help messages ***/
char text1[] = {"
Main Screen:
============

When you start gstalker, you are presented with the main screen. It will try to load the
the group that was active the last time you used gstalker, if there was none, then no charts
will be displayed on start up. The main screen consists of from top to bottom,
pulldown menus, buttonbar, chart stats bar, the chart area, chart data bar.

Pulldown Menu:
--------------
The pulldown menus gives mouse access to all of the functions available in gstalker.

Buttonbar:
----------
The buttonbar gives quick access to most of the major functions in gstalker. Briefly they are
from left to right;

Work with Groups
Work with Charts
Previous Chart
Goto Chart
Next Chart
Compress Chart
Expand Chart
Edit Chart Indicators
Alert Scan
View Data Window
Toggle Chart Grid
Toggle Chart Style
Toggle Volume Indicator

Chart Stats:
------------
This area gives some details on the currently loaded chart. They are;
CH= is the all time high for the data contained in the chart.
CL= is the all time low for the data contained in the chart.
LD= is the last date for the chart.
LC= is the last close value for the chart.
NC= is the net change in value since the previous close.

Chart Area:
-----------
This is where all the chart indicators and plots will be drawn.

Chart Data:
-----------
This is an updated display of the data contained in the chart relative to the position
of the mouse inside the chart area. If the volume indicator is active, and the mouse
enters the volume area, you will see the data be displayed for volume. Another feature
is if you left click and hold with the mouse anywhere in the chart area, you will see
the exact price level be displayed in the data display and not the actual close data.
This is useful for gauging the scale of moves on the chart so that you can note stops
and trendlines.\n\n"};

char text2[] = {"
File Menu:
==========

New Group:
----------
This will create a new group in which to associate individual charts. Just enter the name
of the group you would like to create and pressing ok will do it.

Work With Groups:
---------------
This window gives the ability to perform functions that apply to chart groups. There is
a button on the button bar for this function. By highlighting a group from the list you can:

Open:
Open a chart group for viewing. This will load all the charts that are associated with
a group you have created. By default, the first chart in the group will be displayed.
If the group has no charts associated, then an message will be displayed warning you.
The [group name], chart symbol and (chart name) will be displayed in the titlebar window that
gstalker is running in. eg [nasdaq] - redhat (Red Hat Software).

Edit:
This gives you the ability to edit an existing chart group. By clicking this button
an edit window will appear. The edit group window displays on the left, a list of all the
charts in the gstalker database. The list on the right displays the current contents of
the group being edited. The buttons in the middle allow for editing functions. The insert
button will insert the highlighted selection from the chart list and place it above the
highlighted selection in the group contents list. The add button will append the highlighted
selection in the chart list to the end of the group contents list. The delete button will
delete the highlighted selection from the group contents list. The ok button will save the
changes and exit the edit window. The cancel button will not undo any changes, but just
exits without saving changes.

Rename:
This allows you to rename an existing group, by clicking this button, a new window will
appear and let you enter in the new name for the selected group.

Delete:
Obviously, by clicking on this button, the group highlighted in the list will be
permanently deleted from the group list, once done, there is no way to undelete it.

Cancel:
This just exits from the work with groups window, it does not reverse any changes
that may have be made prior to clicking this button.

Work With Charts:
----------------
This window gives the ability to perform functions that apply to individual charts. There is
a button on the button bar for this function. By highlighting a chart from the list you can:

Open:
Opens a single chart for viewing. The chart symbol and (chart name) will be displayed in the
titlebar window that gstalker is running in. eg redhat (Red Hat Software).

Edit:
This gives you the ability to edit the data of an existing chart. By clicking this button
an edit chart window will appear. All fields can be manually edited except the curent record
and date fields. The buttons allow for navigating through the data and saving changes. The
start button brings you to the first record in the chart. The next button advances forward
one record. The back button retreats one record. The end button skips to the last record.
The save button saves the changes made to the current record. The delete button will delete
permanently the current record. The close button will exit without saving changes. If you
make changes to a record and move to another one without saving, the changes will be lost.

Delete:
Obviously, by clicking on this button, the chart highlighted in the list will be
permanently deleted from the chart list, once done, there is no way to undelete it

Cancel:
This just exits from the work with charts window, it does not reverse any changes
that may have be made prior to clicking this button.

Import Data:
------------
You are presented with several choices. Clicking on the 'import from' button will bring up
a file selector, which you use to choose the file you want to import into gstalker.
The 'import field format' gives some choice on the format of the ascii data to be imported.
Secondly, you can choose whether the fields are separated by space or comma. Thirdly, the
format of the date field.

*Note (It does not matter if there are / slashes in the date field as gstalker will
ignore them, but the date field must be at least 8 characters in length not including
any slashes.)

This function reads ASCII data from a file and converts it to binary, then either creates
or updates a chart in the gstalker database. This is the only way to display data in gstalker.

ASCII is the only file format gstalker understands in this release.

If any errors during import are encountered, they will be written to the error log which
you can view for details with any text editor. The error log is located in the gstalker
directory as error.log.

Export Data:
------------
This feature allows you to export data in gstalker from binary format to ASCII format, so
it can be used in spreadsheets, printed out etc.

An export options window will be displayed. Here you choose from a list of all charts in
the gstalker database to export. Similar to the import options, you can specify how you
want the data to be exported. The default file to create is /tmp/export.txt, click
the Export-> button if you want to browse files, or you can enter a new file name
in the entry space provided.

Quit:
-----
This cleanly exits gstalker and saves any configuration changes that may have been made.
Use this way of terminating the program, if you kill it via the window manager, changes
may not be saved.\n\n"};

char text3[] = {"
Edit Menu:
==========

Edit Chart Indicators:
----------------
This window allows you to add or delete any technical indicators and alerts for the current
chart displayed in the main window. You may leave this window open all the time, so that you
have quick and easy access to the indicators for each active chart. Please be advised that
every time a new chart is displayed, it will re-load the indicator window and wipe out
any unsaved changes. Just remember to save you changes before flipping to the next chart.
There is a button on the button bar for this function.

Indicator Page:
gstalker has two technical indicators, volume and moving average. Any changes saved are
displayed on the chart immediately.

The volume indicator is enabled or disabled by clicking the enable check button.

Moving average is enabled or disabled by checking the enabled button and entering the
moving average length you want. eg entering 10 will create a 10 day moving average.

Alerts Page:
gstalker has three alert indicators. They operate the same way as the technical indicators.
The alerts do not go into effect until an alert scan is performed.

The percent net change alert is enabled or disabled by clicking the enable check button.
Then enter a value eg. entering 60.75 will trigger an alert if the close has gone up 60.75%
from the previous close, or entering -60.75 will trigger the alert if the close has gone
down 60.75% from the previous close.

The percent volume change alert is enabled or disabled by clicking the enable check button.
Then enter a value eg. entering 60.75 will trigger an alert if the volume has gone up 60.75%
from the previous close, or entering -60.75 will trigger the alert if the volume has gone down
60.75% from the previous close.

The threshold alert is enabled or disabled by clicking the enable check button.
Then enter a value eg. entering 60.75 will trigger an alert if the close has penetrated the
60.75 level from the previous close. This is a one-way alert, the alert will be triggered
if it crosses the threshold value either going up or down. This is used to monitor prices
when they break out of sideways price trends.

Edit Options:
-------------
This is where you can customize some of the general features in gstalker.
Pressing the OK button saves the changes and exits the window.

Status Bars Page:
Enable status bars just enables or disables the chart stats and data areas, if disabled,
more space is created to display charts.

Stats bar field enables or disables each individual stat you want displayed.

Data bar field enables or disables each individual data field you want displayed.

Charts Page:
This specifies the maximum amount of data to load for every chart. eg. 500 would load a maximum
of 500 plot points of data. If 0 is entered then the whole chart will be loaded.\n\n"};

char text4[] = {"
View Menu:
==========

Expand Chart:
-------------
This will increase the spacing between the plot points on the chart by one pixel, changing
the scale at which the data appears on the chart. The maximum you can expand is 20 pixels.
There is a button on the button bar for this function.

Compress Chart:
-------------
This will decrease the spacing between the plot points on the chart by one pixel, changing
the scale at which the data appears on the chart.
If OHLC style is active, there is a minimum spacing limitation imposed so the chart is still
legible. There is a button on the button bar for this function.

Line Style:
-----------
This draws the price plot on the chart as an uninterrupted line. There is a button on the
button bar for this function.

OHLC Style:
----------
This draws the price plot on the chart as an open/high/low/close series of ticks and lines.
Where the tick to the left of the high/low range is the open level, the vertical line is the
range of the high/low levels, and the close is the tick to the right of the high/low range.
There is a button on the button bar for this function.

Toggle Chart Grid:
------------------
This will display the background grid in the chart area.

Toggle Volume Indicator:
------------------------
This will toggle the volume indicator as the default for all charts displayed.

View Data Window:
------------------
This will display in a separate window, all the data in the chart in detail form. The window
can be left active at all times, and will be updated whenever a chart is displayed. There
is also the function of left clicking anywhere on the active chart, will immediately highlight
the associated data in the data window if it is active. There is a button on the button bar
for this function.

Goto Chart:
-----------
This allows you to display any chart that is in the currently loaded group. Just highlight
the chart you wish to view and press the OK button. Useful if you have a lot of charts in
a group and do not want to advance to it one at a time using the left and right buttons on
the button bar. There is a button on the button bar for this function.\n\n"};

char text5[] = {"
Tools:
======

Alert Scan:
-----------
This function allows you to scan each chart in the database, and trigger any alerts that have
been set for it. There is a button on the button bar for this function.

The Select Groups To Scan window will display which allows you to choose either a particular
group to be scanned, or you can scan the entire database by enabling the scan all charts
check button. The scan date is the date you want to scan backwards from. eg 19980715 will
scan from July 15, 1998 to the previous date, presumably July 14, 1998 and trigger any alerts
accordingly.

Once the scan is started, the Alert View report window will display a list of any alerts
that have been triggered with a short detail of the alert. Highlighting an alert and pressing
the Goto button will immediately display that chart that triggered the alert. Pressing the
Close button will exit the Alert View window.\n\n"};


/*** flags ***/
int chartflag = 0;
int styleflag = 0;
int open_file_flag = 0;
int data_window_flag = 0;
int volume = 0;
int moving = 0;
int alert_window_flag = 0;
int grid_flag = 0;
int volume_button_flag = 0;
int pixelspace = 3;
int date_flag = 1;
int open_flag = 1;
int high_flag = 1;
int low_flag = 1;
int close_flag = 1;
int volume_flag = 1;
int openi_flag = 1;
int top_enable_flag = 1;
int bottom_enable_flag = 1;
int contract_high_flag = 1;
int contract_low_flag = 1;
int last_date_flag = 1;
int last_close_flag = 1;
int net_change_flag = 1;
int edit_indicators_flag = 0;
int edit_options_flag = 0;
int workwith_groups_flag = 0;
int rename_group_flag = 0;
int new_group_flag = 0;
int edit_group_flag = 0;
int goto_chart_flag = 0;
int workwith_charts_flag = 0;
int edit_chart_flag = 0;
int openfile_flag = 0;
int alert_scan_flag = 0;
int message_flag = 0;
int import_options_flag = 0;
int export_options_flag = 0;
int help_window_flag = 0;
int yesno_flag = 0;

/*** int ***/
int edit_group_size;
int group_pointer;
int group_size;
int entries;
int current_width;
int current_height;
int red, green, blue;
int bars;
int global_int;
int helpdata_size;

/*** char ***/
char name[NAME_SIZE];
char symbol[SYMBOL_SIZE];
char *global_string;
char *grouppath;
char *datapath;
char *errorlog;
char *groupfile_path;
char *groupfilename;
char *importfile;
char *lastdir;
char *error_message;
char *edit_groupfilename;
char *date_alert;
char *config_file;
char edit_symbol[SYMBOL_SIZE];
char edit_name[NAME_SIZE];
char *last_group;
char *helpfile;


/*** long ***/
unsigned long *date_array;
unsigned long *volume_array;
unsigned long *openint_array;
unsigned long average_volume = 0;
unsigned long volume_high = 0;
unsigned long volume_low = 0;
unsigned long *volume_high_pointer;
unsigned long *volume_low_pointer;
unsigned long edit_record;
unsigned long edit_date;


/*** float ***/
float *open_array;
float *high_array;
float *low_array;
float *close_array;
float maxhigh;
float maxlow;
float range;
float volumeroc = 0.0;
float closeroc = 0.0;
float threshold = 0.0;



/*** widget ***/
GtkWidget *main_window;
GtkWidget *message_window;
GtkWidget *data_window;
GtkWidget *data_window_clist;
GtkWidget *data_button;
GtkWidget *alert_window;
GtkWidget *alert_window_clist;
GtkWidget *ticker_combo_box;
GList *list = NULL;
GList *data_list = NULL;
GdkPixmap *pixmap = NULL;
GdkPixmap *pixmap2 = NULL;
GtkWidget *chart;
GtkWidget *scale;
GtkWidget *volume_button;
GtkWidget *grid_button;
GtkWidget *style_button;
GtkWidget *load_upto_button;
GtkWidget *data_frame;
GtkWidget *scrolled_win;
GtkWidget *info_frame;
GtkWidget *info_bar;
GtkWidget *data_bar;
GtkWidget *openfile;
GtkWidget *progwindow;

/*** new group ***/
GtkWidget *new_group_window, *new_group_value_entry;

/*** workwith groups window ***/
GtkWidget *workwith_groups_window, *workwith_groups_clist;

/*** rename group window ***/
GtkWidget *rename_group_window, *rename_group_value_entry;

/*** edit group window ***/
GtkWidget *edit_group_window, *edit_group_clist, *edit_group_clist2;

/*** edit idiactor window ***/
GtkWidget *edit_indicator_window, *enable_volume_ind_check, *enable_moving_ind_check;
GtkWidget *moving_value_entry, *enable_pnc_check, *enable_pvc_check, *enable_threshold_check;
GtkWidget *pnc_value_entry, *pvc_value_entry, *threshold_value_entry;

/*** edit options window ***/
GtkWidget *edit_options_window, *enable_top_check, *enable_bottom_check;
GtkWidget *enable_ch_check, *enable_cl_check, *enable_ld_check;
GtkWidget *enable_lc_check, *enable_nc_check, *enable_date_check;
GtkWidget *enable_open_check, *enable_high_check, *enable_low_check;
GtkWidget *enable_close_check, *enable_volume_check, *enable_openi_check;
GtkWidget *bars_value_entry;

/*** goto chart window ***/
GtkWidget *goto_chart_window, *goto_chart_clist;

/*** workwith charts window ***/
GtkWidget *workwith_charts_window, *workwith_charts_clist;

/*** edit chart window ***/
GtkWidget *edit_chart_open_value_entry, *edit_chart_high_value_entry, *edit_chart_low_value_entry;
GtkWidget *edit_chart_close_value_entry, *edit_chart_volume_value_entry;
GtkWidget *edit_chart_openi_value_entry, *edit_chart_date_statusbar, *edit_chart_record_statusbar;
GtkWidget *edit_chart_window, *edit_chart_name_value_entry;

/*** alert scan window ***/
GtkWidget *alert_scan_window, *alert_scan_clist, *alert_scan_entry, *alert_scan_check;

/*** import options window ***/
GtkWidget *import_options_window, *import_options_radio, *import_options_radio2;
GtkWidget *import_options_radio3, *import_options_radio4, *import_options_radio5;
GtkWidget *import_options_radio6, *import_options_radio7, *import_options_entry;
GtkWidget *import_options_radio8, *import_options_radio9, *import_options_radio10;

/*** export options window ***/
GtkWidget *export_options_window, *export_options_radio, *export_options_radio2;
GtkWidget *export_options_radio3, *export_options_radio4, *export_options_radio5;
GtkWidget *export_options_radio6, *export_options_radio7, *export_options_clist;
GtkWidget *export_options_radio8, *export_options_radio9, *export_options_radio10;
GtkWidget *export_options_entry;

/*** help window ***/
GtkWidget *help_window, *help_text;

/*** yesno window ***/
GtkWidget *yesno_window;

/*** Initialize Directories ***/
GtkWidget *init_window;


/*** prototypes ***/
void exit_program();
void new_group();
void workwith_groups();
void workwith_charts();
void import_options();
void edit_options();
void edit_indicators();
void expand_chart();
void shrink_chart();
void line_style();
void hlc_style();
void show_data_window();
void goto_chart();
void alert_scan();
void about();
void help_index();
void init();
void previous_chart();
void next_chart();
void edit_indicators();
void toggle_grid();
void style_button_pressed();
void toggle_volume();
void expose_event();
void configure_event();
void motion_notify_event();
void scale_expose_event();
void scale_configure_event();
void load_first_group();
void button_press_event();
void export_options();
void new_init();

/**********************************************************************************/
int main (int argc, char *argv[])
{
	GtkWidget *temp, *temp2, *temp3, *vbox, *hbox, *menubar, *toolbar, *pixmapwid;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GtkStyle *style;
	int tint;
	char tstring[250];
	
	
	gtk_init(&argc, &argv);
	strcpy(tstring, getenv("HOME"));
	strcat(tstring, "/gstalker");
	tint = chdir(tstring);
	if (tint != 0)
		tint = 1;
	else
	{
		init();
		tint = 0;
	}
		
	main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_usize(GTK_WIDGET (main_window), 640, 480);
	gtk_signal_connect (GTK_OBJECT (main_window), "delete_event",
			    GTK_SIGNAL_FUNC (exit_program), NULL);
	gtk_signal_connect (GTK_OBJECT (main_window), "destroy",
			    GTK_SIGNAL_FUNC (exit_program), NULL);
	gtk_window_set_title (GTK_WINDOW (main_window), "gstalker");
	gtk_window_position(GTK_WINDOW (main_window), GTK_WIN_POS_CENTER);
	vbox = gtk_vbox_new (FALSE, 0);
      	gtk_container_add (GTK_CONTAINER (main_window), vbox);
      	gtk_widget_show (vbox);
      	
      	/*** create the menubar ***/
      	
      	menubar = gtk_menu_bar_new ();
        gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, TRUE, 0);
        gtk_widget_show (menubar);
        
        temp = gtk_menu_new ();
        temp2 = gtk_menu_item_new_with_label ("File");
        gtk_widget_show (temp2);
        
        temp3 = gtk_menu_item_new_with_label("New Group...");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (new_group), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Work With Groups...");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (workwith_groups), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Work With Charts...");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (workwith_charts), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new();
        gtk_menu_append (GTK_MENU (temp), temp3);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Import Data...");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (import_options), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Export Data...");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (export_options), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new();
        gtk_menu_append (GTK_MENU (temp), temp3);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Quit");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (exit_program), NULL);
	gtk_widget_show (temp3);
	
        gtk_menu_item_set_submenu (GTK_MENU_ITEM (temp2), temp);
      	gtk_menu_bar_append (GTK_MENU_BAR (menubar), temp2);
      	
      	temp = gtk_menu_new ();
        temp2 = gtk_menu_item_new_with_label ("Edit");
        gtk_widget_show (temp2);
        
        temp3 = gtk_menu_item_new_with_label("Edit Chart indicators...");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (edit_indicators), NULL);
	gtk_widget_show (temp3);
        
	temp3 = gtk_menu_item_new_with_label("Edit options...");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (edit_options), NULL);
	gtk_widget_show (temp3);
	
        gtk_menu_item_set_submenu (GTK_MENU_ITEM (temp2), temp);
      	gtk_menu_bar_append (GTK_MENU_BAR (menubar), temp2);
      	
      	temp = gtk_menu_new ();
        temp2 = gtk_menu_item_new_with_label ("View");
        gtk_widget_show (temp2);
        
        temp3 = gtk_menu_item_new_with_label("Expad Chart");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (expand_chart), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Compress Chart");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (shrink_chart), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new();
        gtk_menu_append (GTK_MENU (temp), temp3);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Previous Chart");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (previous_chart), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Next Chart");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (next_chart), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new();
        gtk_menu_append (GTK_MENU (temp), temp3);
	gtk_widget_show (temp3);
		
        temp3 = gtk_menu_item_new_with_label("Line Style");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (line_style), NULL);
	gtk_widget_show (temp3);
	
        temp3 = gtk_menu_item_new_with_label("OHLC Style");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (hlc_style), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Toggle Chart Grid");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (toggle_grid), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new();
        gtk_menu_append (GTK_MENU (temp), temp3);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Toggle Volume indicator");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (toggle_volume), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new();
        gtk_menu_append (GTK_MENU (temp), temp3);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("View Data window");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (show_data_window), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new();
        gtk_menu_append (GTK_MENU (temp), temp3);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Goto Chart...");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (goto_chart), NULL);
	gtk_widget_show (temp3);
	
        gtk_menu_item_set_submenu (GTK_MENU_ITEM (temp2), temp);
      	gtk_menu_bar_append (GTK_MENU_BAR (menubar), temp2);
      	
	temp = gtk_menu_new ();
        temp2 = gtk_menu_item_new_with_label ("Tools");
        gtk_widget_show (temp2);
        
        temp3 = gtk_menu_item_new_with_label("Alert Scan");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (alert_scan), NULL);
	gtk_widget_show (temp3);
	
        gtk_menu_item_set_submenu (GTK_MENU_ITEM (temp2), temp);
      	gtk_menu_bar_append (GTK_MENU_BAR (menubar), temp2);
      	
      	temp = gtk_menu_new ();
        temp2 = gtk_menu_item_new_with_label ("Help");
        gtk_widget_show (temp2);
        
        temp3 = gtk_menu_item_new_with_label("About...");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (about), NULL);
	gtk_widget_show (temp3);
	
	temp3 = gtk_menu_item_new_with_label("Help...");
        gtk_menu_append (GTK_MENU (temp), temp3);
        gtk_signal_connect (GTK_OBJECT (temp3), "activate",
			    GTK_SIGNAL_FUNC (help_index), NULL);
	gtk_widget_show (temp3);
	
        gtk_menu_item_set_submenu (GTK_MENU_ITEM (temp2), temp);
      	gtk_menu_bar_append (GTK_MENU_BAR (menubar), temp2);
      	
      	gtk_widget_realize( main_window );

	/*** create the toolbar ***/
	
	style = gtk_widget_get_style(main_window);
	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
	gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar), 10);
	gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, TRUE, 2);
	
    	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Tb_open_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
    	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Work With Groups",
    				 NULL, pixmapwid, (GtkSignalFunc) workwith_groups, NULL);
    				 
    	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Tb_new_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
    	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Work With Charts",
    				 NULL, pixmapwid, (GtkSignalFunc) workwith_charts, NULL);
    				 
	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
	
    	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Tb_left_arrow_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
    	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Previous Chart",
    				 NULL, pixmapwid, (GtkSignalFunc) previous_chart, NULL);
    				 
    	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Tb_refresh_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
    	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Goto Chart",
    				 NULL, pixmapwid, (GtkSignalFunc) goto_chart, NULL);
    				 
    	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Tb_right_arrow_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Next Chart",
    				 NULL, pixmapwid, (GtkSignalFunc) next_chart, NULL);
    				 
	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
	
	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Minus_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Compress Chart",
    				 NULL, pixmapwid, (GtkSignalFunc) shrink_chart, NULL);
    				 
    	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Plus_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
    	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Expand Chart",
   				 NULL, pixmapwid, (GtkSignalFunc) expand_chart, NULL);
    				 
	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
	
	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Tb_preferences_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Edit Chart indicators",
    				 NULL, pixmapwid, (GtkSignalFunc) edit_indicators, NULL);
    	
	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Tb_search_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Alert Scan",
    				 NULL, pixmapwid, (GtkSignalFunc) alert_scan, NULL);
    				 
    	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
    				 
	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Info_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
    	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "View Data window",
    				 NULL, pixmapwid, (GtkSignalFunc) show_data_window, NULL);
    				 
    	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Grid_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
    	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Toggle Grid",
    				 NULL, pixmapwid, (GtkSignalFunc) toggle_grid, NULL);
    	
    	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Line_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
    	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Toggle Chart Style",
    				 NULL, pixmapwid, (GtkSignalFunc) style_button_pressed, NULL);
    	
	pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    (gchar **) Dial_xpm);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
    	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), NULL, "Toggle Volume indicator",
    				 NULL, pixmapwid, (GtkSignalFunc) toggle_volume, NULL);
    				 
    	gtk_widget_show (toolbar);
    	
    	/**** info bar ****/
	
	info_bar = gtk_hbox_new (FALSE, 0);
    	gtk_box_pack_start (GTK_BOX (vbox), info_bar, FALSE, TRUE, 0);
      	gtk_widget_show (info_bar);    	
    	if (top_enable_flag)
    	{
      		info_frame = gtk_statusbar_new ();
      		gtk_box_pack_start (GTK_BOX (info_bar), info_frame, TRUE, TRUE, 2);
      		gtk_widget_show (info_frame);
      		gtk_widget_realize(info_frame);
      	}
    	
    	/*** chart area ***/
    	
    	hbox = gtk_hbox_new (FALSE, 0);
      	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
      	gtk_widget_show (hbox);
    	
      	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
      	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
				      	GTK_POLICY_AUTOMATIC, 
				      	GTK_POLICY_AUTOMATIC);
      	gtk_box_pack_start (GTK_BOX (hbox), scrolled_win, TRUE, TRUE, 0);
      	gtk_container_border_width (GTK_CONTAINER (scrolled_win), 0);
      	gtk_widget_show (scrolled_win);
      	
      	chart = gtk_drawing_area_new();
      	gtk_signal_connect (GTK_OBJECT(chart), "expose_event",
      			    (GtkSignalFunc) expose_event, NULL);
      	gtk_signal_connect (GTK_OBJECT(chart), "configure_event",
      			    (GtkSignalFunc) configure_event, NULL);		    
      	gtk_signal_connect (GTK_OBJECT(chart), "motion_notify_event",
      			    (GtkSignalFunc) motion_notify_event, NULL);
      	gtk_signal_connect (GTK_OBJECT(chart), "button_press_event",
      			    (GtkSignalFunc) button_press_event, NULL);
      	gtk_widget_set_events (chart, GDK_EXPOSURE_MASK
      			       | GDK_LEAVE_NOTIFY_MASK
      			       | GDK_BUTTON_PRESS_MASK
      			       | GDK_POINTER_MOTION_MASK
      			       | GDK_POINTER_MOTION_HINT_MASK);
      	gtk_container_add (GTK_CONTAINER (scrolled_win), chart);
      	gtk_widget_show(chart);
      	
      	scale = gtk_drawing_area_new();
      	gtk_signal_connect (GTK_OBJECT(scale), "expose_event",
      			    (GtkSignalFunc) scale_expose_event, NULL);
      	gtk_signal_connect (GTK_OBJECT(scale), "configure_event",
      			    (GtkSignalFunc) scale_configure_event, NULL);		    
      	gtk_widget_set_events (scale, GDK_EXPOSURE_MASK);
      	gtk_widget_set_usize(GTK_WIDGET (scale), WIDTH_OFFSET, 0);
      	gtk_box_pack_end (GTK_BOX (hbox), scale, FALSE, FALSE, 0);
      	gtk_widget_show(scale);
      	
      	/*** data_bar setup ***/
      	
      	data_bar = gtk_hbox_new (FALSE, 0);
    	gtk_box_pack_start (GTK_BOX (vbox), data_bar, FALSE, TRUE, 0);
      	gtk_widget_show (data_bar);
      	if (bottom_enable_flag)
      	{
      		data_frame = gtk_statusbar_new ();
      		gtk_box_pack_start (GTK_BOX (data_bar), data_frame, TRUE, TRUE, 2);
      		gtk_widget_show (data_frame);
      	}
 	
	gtk_widget_show(main_window);
	if (! tint)
	{
		if (strlen(last_group) != 0)
 			load_first_group();
 	}
 	else
 		new_init();		
	gtk_main();
	return 0;							
}